@@ -16,11 +16,6 @@ correlations <- function(simulations,
     simulations$angle.count$id <- as.numeric(as.factor(simulations$angle.count$id))
 
 
-
-  if("W" %in% colnames(simulations$fixed.area))
-    variables = c(variables, "W")
-
-
   # Define a character vector containing index name (radius, k or BAF) for each
   # available plot design
   .plot.design <- c(fixed.area = "radius", k.tree = "k", angle.count = "BAF")
@@ -31,9 +26,9 @@ correlations <- function(simulations,
     # Density (trees/ha), basal area (m2/ha) and volume (m3/ha)
     "N", "G", "V",
 
-    # Biomass (Mg/ha)
-    if("W" %in% colnames(simulations$fixed.area))
-      "W",
+    # Volume (m3/ha) and biomass (Mg/ha) provided by user
+    # if("W" %in% colnames(simulations$fixed.area))
+    "V.user", "W.user",
 
     # Mean diameters (cm), and mean dominant diameters (cm)
     "d", "dg", "dgeom", "dharm",
@@ -541,8 +536,8 @@ correlations <- function(simulations,
             switch(.k, N = "Density (N, trees/ha)",
                    G = "Basal area (G, m<sup>2</sup>/ha)",
                    V = "Volume (V, m<sup>3</sup>/ha)",
-                   if("W" %in% colnames(simulations$fixed.area))
-                     W = "Biomass (W, Mg/ha)",
+                   V.user = "Volume (V, m<sup>3</sup>/ha) provided by user",
+                   W.user = "Biomass (W, Mg/ha) provided by user",
                    d = "Arithmetic mean diameter (d, cm)",
                    dg = "Quadratic mean diameter (dg, cm)",
                    dgeom = "Geometric mean diameter (dgeom, cm)",
